
#ifndef _NET_H
#define _NET_H


#include <stdint.h>
#include "hash.h"

/* some useful typedefs */
typedef uint8_t 	u8;
typedef uint16_t 	u16;
typedef uint32_t	u32;
typedef uint64_t	u64;


/* network operations and structures */

/* request opcodes */
#define REQ_ACQ_LOCK		1
#define REQ_REL_LOCK		2
#define REQ_TRY_LOCK		3
#define REQ_PING		4
#define REQ_ADOPT		5
#define REQ_SYNC		6

/* reply opcodes */
#define REP_LOCK_ACQUIRED	128
#define REP_LOCK_WBLOCK		129
#define REP_LOCK_RELEASED	130
#define REP_PONG		131
#define REP_ACK			132
#define REP_ERR			133
#define REP_SYNC		134

/* max payload size, (2^20 - 1) */
#define MAX_PAYLOAD (1048575)

/* main command structure */
struct net_cmd {
	u32 len;
	u8 ver;
	u8 op;
	char *payload;
};

/* functions */
void net_main_loop();
int net_send_cmd(int fd, unsigned int op, char *payload, int len);
struct net_cmd *net_get_cmd(int fd);
int net_close(int fd);
int net_parse(int fd, struct net_cmd *cmd);
int net_wakeup(struct hentry *h);
void clean_orphans();
void clean_buffer(int fd);

#endif

